/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef OVS_DOCA_H_
#define OVS_DOCA_H_

#include <config.h>

#include "dpif-netdev.h"
#include "smap.h"
#include "vswitch-idl.h"

#if DOCA_OFFLOAD
#include <doca_flow.h>

#define OVS_DOCA_MAX_CT_CONNS 250000

/* Connections are offloaded with one hardware rule per direction.
 * The netdev-offload layer manages offloads rule-wise, so a
 * connection is handled in two parts. This discrepancy can be
 * misleading.
 * This macro expresses the number of hardware rules required
 * to handle the number of CT connections supported by ovs-doca.
 */
#define OVS_DOCA_MAX_CT_RULES (OVS_DOCA_MAX_CT_CONNS * 2)

/* DOCA requrires everything upfront. As a WA we define max number of ESW. */
#define OVS_DOCA_MAX_ESW    2

/* Using shared counters we need 1 per connection */
#define OVS_DOCA_MAX_CT_COUNTERS_PER_ESW OVS_DOCA_MAX_CT_CONNS
#define OVS_DOCA_MAX_CT_COUNTERS \
    (OVS_DOCA_MAX_CT_COUNTERS_PER_ESW * OVS_DOCA_MAX_ESW)

#define OVS_DOCA_QUEUE_DEPTH 32

#define OVS_DOCA_MAX_METERS_PER_ESW MAX_METERS
#define OVS_DOCA_MAX_METERS (OVS_DOCA_MAX_METERS_PER_ESW * OVS_DOCA_MAX_ESW)

extern bool ovs_doca_async;
extern uint32_t ctl_pipe_size;

void
ovs_doca_entry_process_cb(struct doca_flow_pipe_entry *entry, uint16_t qid,
                          enum doca_flow_entry_status status,
                          enum doca_flow_entry_op op, void *aux);
#endif /* DOCA_OFFLOAD */

struct ofputil_meter_config;
struct rte_mtr_error;

bool
ovs_doca_enabled(void);

int
ovs_doca_init(const struct smap *ovs_other_config);

void *
ovs_doca_port_create(uint16_t port_id);

int
ovs_doca_port_destroy(void *port);

void
ovs_doca_status(const struct ovsrec_open_vswitch *cfg);

void
ovs_doca_destroy(void);

void
print_doca_version(void);

int
ovs_doca_create_meter(uint32_t meter_id,
                      struct ofputil_meter_config *config,
                      struct rte_mtr_error *error);
#endif
